==> crunch_uml-0.2.5.tar.gz: crunch_uml-0.2.5/crunch_uml/parsers/multiple_parsers.py <==
import json
import logging
import os

import pandas as pd
import requests

import crunch_uml.schema as sch
from crunch_uml import db
from crunch_uml.excpetions import CrunchException
from crunch_uml.parsers.parser import Parser, ParserRegistry

logger = logging.getLogger()


def store_data(entity_name, data, schema):
    # session = SessionLocal()
    # Retrieve all models dynamically
    base = db.Base
    session = schema.get_session()

    logger.debug(f"Parsing {len(data)} entities with name {entity_name}")
    # Model class associated with the table
    entity = base.model_lookup_by_table_name(entity_name)
    if not entity:
        logger.warning(f"Entity not found with tablename: {entity_name}.")
        return

    # Als er een ID in de data aanwezig is, zoek dan naar een bestaand record
    if "id" in data:
        if existing_entity := session.query(entity).filter_by(id=data["id"], schema_id=schema.schema_id).first():
            for key, value in data.items():
                if value is not None and value != '':
                    setattr(existing_entity, key, value)
            logger.debug(f"Updated {entity_name} with ID {data['id']}.")
            schema.save(existing_entity)
        else:
            # ID was aanwezig, maar geen overeenkomstige record werd gevonden
            logger.debug(f"No {entity_name} found with ID {data['id']}, creating a new record.")
            new_entity = entity(**data)
            schema.save(new_entity)
    else:
        logger.warning(
            f"Could not save entity with table '{entity_name}' to schema {schema.schema_id}: no imakd present: {data}."
        )


@ParserRegistry.register(
    "json", descr='Generic parser that parses JSON-files, and looks for table and column definitions.'
)
class JSONParser(Parser):
    def parse(self, args, schema: sch.Schema):
        logger.info(f"Starting parsing JSON file {args.inputfile}")
        # sourcery skip: raise-specific-error
        try:
            if args.inputfile is not None:
                with open(args.inputfile, 'r') as f:
                    parsed_data = json.load(f)
            elif args.url is not None:
                response = requests.get(args.url)
                response.raise_for_status()  # Zorg dat we een fout krijgen als de download mislukt
                parsed_data = response.json()  #

            tables = db.getTables()
            # Ga ervan uit dat het JSON-bestand een structuur heeft zoals eerder beschreven
            for entity_name, records in parsed_data.items():
                if entity_name in tables and entity_name != 'schemas':
                    for record in records:
                        store_data(entity_name, record, schema)
        except json.JSONDecodeError as ex:
            msg = f"File with name {args.inputfile} is not a valid JSON-file, aborting with message {ex.msg}"
            logger.error(msg)
            raise CrunchException(msg) from ex
        logger.info(f"Ended parsing JSON file {args.inputfile} with success")


@ParserRegistry.register(
    "xlsx",
    descr=(
        'Generic parser that parses Excel files, and excpect one or more worksheets that correspond with the names of'
        ' one or more of the tables.'
    ),
)
class XLXSParser(Parser):
    def parse(self, args, schema: sch.Schema):
        # sourcery skip: raise-specific-error
        logger.info(f"Starting parsing Excel file {args.inputfile}")

        try:
            # Lees het Excel-bestand
            xls = pd.ExcelFile(args.inputfile if args.inputfile is not None else args.url)

            tables = db.getTables()
            # Loop door elk tabblad in het Excel-bestand
            for sheet_name in xls.sheet_names:
                if sheet_name in tables and sheet_name != 'schemas':
                    # Lees de gegevens van het huidige tabblad als een lijst van woordenboeken
                    records = xls.parse(sheet_name).to_dict(orient='records')

                    for record in records:
                        store_data(sheet_name, record, schema)

        except Exception as ex:
            msg = f"Error while parsing the Excel file {args.inputfile}: {str(ex)}"
            logger.error(msg)
            raise CrunchException(msg) from ex

        logger.info(f"Ended parsing Excel file {args.inputfile} with success")


@ParserRegistry.register(
    "csv", descr='Generic parser that parses one CSV file, and excpect its name to be in the list of tables.'
)
class CSVParser(Parser):
    def parse(self, args, schema: sch.Schema):
        logger.info(f"Starting parsing CSV file {args.inputfile}")

        try:
            # Haal de entiteitsnaam uit de bestandsnaam (verwijder het .csv-deel)
            entity_name = os.path.splitext(os.path.basename(args.inputfile))[0]

            tables = db.getTables()
            if entity_name in tables and entity_name != 'schemas':
                # Lees het CSV-bestand in een dataframe
                df = pd.read_csv(args.inputfile if args.inputfile is not None else args.url)

                # Converteer het dataframe naar een lijst van woordenboeken (records)
                records = df.to_dict(orient='records')

                for record in records:
                    store_data(entity_name, record, schema)

            else:
                logger.warning(f"Could not import file: no entity found with name {entity_name}")

        except Exception as ex:
            msg = f"Error while parsing the CSV file {args.inputfile}: {str(ex)}"
            logger.error(msg)
            raise CrunchException(msg) from ex

        logger.info(f"Ended parsing CSV file {args.inputfile} with success")

==> crunch_uml-0.2.14.tar.gz: crunch_uml-0.2.14/crunch_uml/parsers/multiple_parsers.py <==
import json
import logging
import os

import pandas as pd
import requests

import crunch_uml.schema as sch
from crunch_uml import const, db
from crunch_uml.exceptions import CrunchException
from crunch_uml.parsers.parser import Parser, ParserRegistry

logger = logging.getLogger()


def store_data(entity_name, data, schema, update_only=False):
    # session = SessionLocal()
    # Retrieve all models dynamically
    base = db.Base
    session = schema.get_session()

    logger.debug(f"Parsing {len(data)} entities with name {entity_name}")
    # Model class associated with the table
    entity = base.model_lookup_by_table_name(entity_name)
    if not entity:
        logger.warning(f"Entity not found with tablename: {entity_name}.")
        return

    # Als er een ID in de data aanwezig is, zoek dan naar een bestaand record
    if "id" in data:
        if existing_entity := session.query(entity).filter_by(id=data["id"], schema_id=schema.schema_id).first():
            for key, value in data.items():
                if value is not None and value != "" and key != "id":
                    setattr(existing_entity, key, value)
            logger.debug(f"Updated {entity_name} with ID {data['id']}.")
            schema.save(existing_entity)
        else:
            if not update_only:
                # ID was aanwezig, maar geen overeenkomstige record werd gevonden
                logger.debug(f"No {entity_name} found with ID {data['id']}, creating a new record.")
                new_entity = entity(**data)
                schema.save(new_entity)
    else:
        if not update_only:
            logger.debug(
                f"Could not save entity with table '{entity_name}' to schema {schema.schema_id}: no column id present:"
                f" {data}."
            )
            new_entity = entity(**data)
            schema.save(new_entity)


@ParserRegistry.register(
    "json",
    descr="Generic parser that parses JSON-files, and looks for table and column definitions.",
)
class JSONParser(Parser):

    def update_only(self):
        return False

    def get_data_subset(self, data, args):
        return data

    def transform_record(self, record):
        """Update or insert the record in the database"""
        return record

    def parse(self, args, schema: sch.Schema):
        logger.info(f"Starting parsing JSON file {args.inputfile}")
        # sourcery skip: raise-specific-error
        try:
            if args.inputfile is not None:
                with open(args.inputfile, "r") as f:
                    parsed_data = json.load(f)
            elif args.url is not None:
                response = requests.get(args.url)
                response.raise_for_status()  # Zorg dat we een fout krijgen als de download mislukt
                parsed_data = response.json()  #
            parsed_data = self.get_data_subset(parsed_data, args)

            tables = db.getTables()
            # Ga ervan uit dat het JSON-bestand een structuur heeft zoals eerder beschreven
            for entity_name, records in parsed_data.items():
                if entity_name in tables and entity_name != "schemas":
                    for record in records:
                        record = self.transform_record(record)
                        store_data(entity_name, record, schema, update_only=self.update_only())
        except json.JSONDecodeError as ex:
            msg = f"File with name {args.inputfile} is not a valid JSON-file, aborting with message {ex.msg}"
            logger.error(msg)
            raise CrunchException(msg) from ex
        logger.info(f"Ended parsing JSON file {args.inputfile} with success")


@ParserRegistry.register(
    "i18n",
    descr=f"Parser that reads i18n file and stores the values in the database. Use --language to specify language. (default: {const.DEFAULT_LANGUAGE})",
)
class I18nParser(JSONParser):

    def update_only(self):
        """Update only the records that already exist. do not add new ones to not get errors"""
        return True

    def get_data_subset(self, data, args):
        # Bepaal welke taal moet worden verwerkt
        language = args.language if args.language else const.LANGUAGE.DEFAULT
        if language not in data:
            raise ValueError(f"Language '{language}' not found in the i18n file.")

        return data[language]

    def transform_record(self, record):
        # i18n records always are in the form of RECORD_TYPE_INDEXED: key: {record}
        key, value = next(iter(record.items()))
        record = value
        record["id"] = key
        return record


@ParserRegistry.register(
    "xlsx",
    descr=(
        "Generic parser that parses Excel files, and excpect one or more worksheets that correspond with the names of"
        " one or more of the tables."
    ),
)
class XLXSParser(Parser):
    def parse(self, args, schema: sch.Schema):
        # sourcery skip: raise-specific-error
        logger.info(f"Starting parsing Excel file {args.inputfile}")

        try:
            # Lees het Excel-bestand
            xls = pd.ExcelFile(args.inputfile if args.inputfile is not None else args.url)

            tables = db.getTables()
            # Loop door elk tabblad in het Excel-bestand
            for sheet_name in xls.sheet_names:
                if sheet_name in tables and sheet_name != "schemas":
                    # Lees de gegevens van het huidige tabblad als een lijst van woordenboeken
                    records = xls.parse(sheet_name).to_dict(orient="records")

                    for record in records:
                        store_data(sheet_name, record, schema)

        except Exception as ex:
            msg = f"Error while parsing the Excel file {args.inputfile}: {str(ex)}"
            logger.error(msg)
            raise CrunchException(msg) from ex

        logger.info(f"Ended parsing Excel file {args.inputfile} with success")


@ParserRegistry.register(
    "csv",
    descr="Generic parser that parses one CSV file, and excpect its name to be in the list of tables.",
)
class CSVParser(Parser):
    def parse(self, args, schema: sch.Schema):
        logger.info(f"Starting parsing CSV file {args.inputfile}")

        try:
            # Haal de entiteitsnaam uit de bestandsnaam (verwijder het .csv-deel)
            entity_name = os.path.splitext(os.path.basename(args.inputfile))[0]

            tables = db.getTables()
            if entity_name in tables and entity_name != "schemas":
                # Lees het CSV-bestand in een dataframe
                df = pd.read_csv(args.inputfile if args.inputfile is not None else args.url)

                # Converteer het dataframe naar een lijst van woordenboeken (records)
                records = df.to_dict(orient="records")

                for record in records:
                    store_data(entity_name, record, schema)

            else:
                logger.warning(f"Could not import file: no entity found with name {entity_name}")

        except Exception as ex:
            msg = f"Error while parsing the CSV file {args.inputfile}: {str(ex)}"
            logger.error(msg)
            raise CrunchException(msg) from ex

        logger.info(f"Ended parsing CSV file {args.inputfile} with success")
